/**
 * RapidMiner GmbH
 *
 * Copyright (C) 2020 by RapidMiner GmbH and the contributors
 *
 * Complete list of developers available at our web site:
 *
 *      www.rapidminer.com
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 */
package com.rapidminer.extension.connection_conversion.operator;

import static com.rapidminer.extension.connection_conversion.connection.SharedDataConfigurator.CON_TYPE_SHARED_DATA;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.ArrayUtils;

import com.rapidminer.Process;
import com.rapidminer.connection.ConnectionHandlerRegistry;
import com.rapidminer.connection.adapter.ConnectionAdapterHandler;
import com.rapidminer.connection.util.ConnectionInformationSelector;
import com.rapidminer.connection.util.ConnectionSelectionProvider;
import com.rapidminer.example.Attribute;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.DataRow;
import com.rapidminer.example.table.DoubleArrayDataRow;
import com.rapidminer.example.utils.ExampleSets;
import com.rapidminer.extension.connection_conversion.connection.SharedDataConfigurable;
import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.OperatorVersion;
import com.rapidminer.operator.UserError;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.operator.ports.metadata.GenerateNewExampleSetMDRule;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.tools.Ontology;
import com.rapidminer.tools.config.ConfigurationException;
import com.rapidminer.tools.config.ParameterTypeConfigurable;

/**
 * Simple operator that takes a {@link SharedDataConfigurable} as a parameter or input and outputs an {@link ExampleSet} created
 * from {@link SharedDataConfigurable#getConfiguration() the configuration}.
 *
 * @since 1.0.0
 */
public class ReadSharedDataOperator extends Operator implements ConnectionSelectionProvider {

	public static final String PARAMETER_SHARED_DATA = "shared_data";

	private final OutputPort outputPort = getOutputPorts().createPort("example set");
	private ConnectionInformationSelector connectionSelector;

	public ReadSharedDataOperator(OperatorDescription description) {
		super(description);
		getTransformer().addRule(new GenerateNewExampleSetMDRule(outputPort));
		getTransformer().addRule(ConnectionAdapterHandler.createProcessSetupRule(this));
	}

	@Override
	public void doWork() throws OperatorException {
		Process process = getProcess();
		SharedDataConfigurable sharedDataConnection;
		try {
			sharedDataConnection = ConnectionAdapterHandler.getAdapter(this,
					PARAMETER_SHARED_DATA, CON_TYPE_SHARED_DATA,
					process == null ? null : process.getRepositoryAccessor());
		} catch (ConfigurationException e) {
			throw new UserError(this, e, "Configuration error!");
		}
		Map<String, String> configuration = sharedDataConnection.getConfiguration();


		List<Attribute> attributes = new ArrayList<>();
		configuration.forEach((k, v) -> attributes.add(AttributeFactory.createAttribute(k, Ontology.NOMINAL)));
		double[] data = new double[attributes.size()];
		for (int i = 0; i < attributes.size(); i++) {
			Attribute attribute = attributes.get(i);
			data[i] = attribute.getMapping().mapString(configuration.get(attribute.getName()));
		}
		DataRow dataRow = new DoubleArrayDataRow(data);
		ExampleSet output = ExampleSets.from(attributes).addDataRow(dataRow).build();
		outputPort.deliver(output);
	}

	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = new ArrayList<>();
		ParameterTypeConfigurable configurableParameter = new ParameterTypeConfigurable(PARAMETER_SHARED_DATA, "The shared data configuration", CON_TYPE_SHARED_DATA);
		types.addAll(ConnectionAdapterHandler.getConnectionParameters(this, CON_TYPE_SHARED_DATA, configurableParameter));
		return types;
	}

	@Override
	public ConnectionInformationSelector getConnectionSelector() {
		return connectionSelector;
	}

	@Override
	public void setConnectionSelector(ConnectionInformationSelector selector) {
		this.connectionSelector = selector;
	}

	@Override
	public OperatorVersion[] getIncompatibleVersionChanges() {
		return (OperatorVersion[]) ArrayUtils.add(super.getIncompatibleVersionChanges(),
				ConnectionHandlerRegistry.BEFORE_NEW_CONNECTION_MANAGEMENT);
	}
}
